package com.cedarsoft.spring.rcp.async;

import com.cedarsoft.CanceledException;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

/**
 * An actiont hat can be run in background (using a BlockingBackgroundRunner)
 */
public abstract class BackgroundAction {
  @NotNull
  @NonNls
  private final String initialProgressMessageKey;

  /**
   * Creates a new background action
   *
   * @param initialProgressMessageKey the key used for the inital progress message
   */
  protected BackgroundAction( @NotNull @NonNls String initialProgressMessageKey ) {
    this.initialProgressMessageKey = initialProgressMessageKey;
  }

  /**
   * This method is called within the EDT. It is possible to configure the action here
   *
   * @throws CanceledException if the preparation is canceled
   */
  protected void prepare() throws CanceledException {
  }

  /**
   * This method is called asynchroniously
   * @throws Exception
   */
  protected abstract void executeInBackground() throws Exception;

  /**
   * Confirm whether the action shall be started.
   * Is called in EDT
   *
   * @return true if the action shall be started, false otherwise
   */
  protected boolean confirm() {
    return true;
  }

  /**
   * Returns the message key for the initial progress message
   *
   * @return the key
   */
  @NotNull
  @NonNls
  public String getInitialProgressMessageKey() {
    return initialProgressMessageKey;
  }

  /**
   * This method is called (in EDT) when the background action has been finshed
   */
  protected void finished() {
  }
}