package com.cedarsoft.spring.rcp.beans;

import com.cedarsoft.history.ObservableObjectAccess;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.form.binding.Binder;

/**
 * Offers some methods that offer some kind of REST support for a list of beans.
 *
 * @param <T> the type this access agent creates binder for
 */
public interface AccessAgent<T> {
  /**
   * Returns the binder that can be used to bind *one* instance of a bean
   *
   * @param objectAccess the object access that contains the possible values
   * @return the binder
   */
  @NotNull
  Binder getSingleBinder( @NotNull ObservableObjectAccess<T> objectAccess );

  /**
   * Returns a special binder that can be used to bind a collection (ObjectAccess) of a bean type
   *
   * @return the binder
   */
  @NotNull
  Binder getMultiBinder();
}