package com.cedarsoft.spring.rcp.beans;

import org.jetbrains.annotations.NotNull;

/**
 * Default implementation for bean ui access.
 *
 * @param <T> the type
 */
public class DefaultBeanUiAccess<T> implements BeanUiAccess<T> {
  @NotNull
  private final Class<T> type;
  @NotNull
  private final BeanAgent<T> agent;
  @NotNull
  private final BeansDisplay beansDisplay;
  @NotNull
  private final AccessAgent accessAgent;

  /**
   * Creates a new bean ui access with the given objects
   *
   * @param type         the type
   * @param agent        the agent
   * @param accessAgent  the access agent
   * @param beansDisplay the beans display
   */
  public DefaultBeanUiAccess( @NotNull Class<T> type, @NotNull BeanAgent<T> agent, AccessAgent accessAgent, @NotNull BeansDisplay beansDisplay ) {
    this.type = type;
    this.agent = agent;
    this.accessAgent = accessAgent;
    this.beansDisplay = beansDisplay;
  }

  @Override
  @NotNull
  public Class<T> getBeanType() {
    return type;
  }

  @Override
  @NotNull
  public AccessAgent getAccessAgent() {
    return accessAgent;
  }

  @Override
  @NotNull
  public BeanAgent<T> getBeanAgent() {
    return agent;
  }

  @Override
  @NotNull
  public BeansDisplay getBeansDisplay() {
    return beansDisplay;
  }
}
