package com.cedarsoft.spring.rcp.beans;

import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.spring.rcp.binding.ObjectTablePanelMultiBinder;
import com.cedarsoft.spring.rcp.binding.ObjectTablePanelSingleBinder;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanelFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.form.binding.Binder;
import org.springframework.richclient.selection.binding.support.LabelProvider;

/**
 * Default access agent that is based on object tables placed in table based panels.
 *
 * @param <T> the type
 */
public class ObjectTableBasedAccessAgent<T> implements AccessAgent<T> {
  @NotNull
  private final ObjectTablePanelFactory<T> objectTablePanelFactory;
  @NotNull
  private final LabelProvider labelProvider;

  /**
   * Creates a new agent
   *
   * @param labelProvider           the label provider
   * @param objectTablePanelFactory the panel factory
   */
  public ObjectTableBasedAccessAgent( @NotNull LabelProvider labelProvider, @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory ) {
    this.labelProvider = labelProvider;
    this.objectTablePanelFactory = objectTablePanelFactory;
  }

  @Override
  @NotNull
  public Binder getSingleBinder( @NotNull ObservableObjectAccess<T> objectAccess ) {
    return new ObjectTablePanelSingleBinder<T>( objectAccess, labelProvider, objectTablePanelFactory );
  }

  @Override
  @NotNull
  public Binder getMultiBinder() {
    return new ObjectTablePanelMultiBinder<T>( objectTablePanelFactory );
  }
}
