package com.cedarsoft.spring.rcp.binding;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.binding.form.FormModel;
import org.springframework.richclient.form.binding.support.AbstractBinder;
import org.springframework.richclient.selection.binding.support.LabelProvider;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import java.util.HashMap;
import java.util.Map;

/**
 * @param <T> the type
 */
public class LabelBinder<T> extends AbstractBinder {
  @NotNull
  public static <T> LabelBinding bind( @NotNull LabelProvider labelProvider, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath ) {
    return bind( null, labelProvider, formModel, formPropertyPath );
  }

  @NotNull
  public static <T> LabelBinding bind( @NotNull LabelProvider labelProvider, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath, boolean multiLine ) {
    return bind( null, labelProvider, formModel, formPropertyPath, multiLine );
  }

  @NotNull
  public static <T> LabelBinding bind( @Nullable Class<T> type, @NotNull LabelProvider labelProvider, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath ) {
    return bind( type, labelProvider, formModel, formPropertyPath, false );
  }

  @NotNull
  public static <T> LabelBinding bind( @Nullable Class<T> type, @NotNull LabelProvider labelProvider, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath, boolean multiLine ) {
    LabelBinder<T> binder = new LabelBinder<T>( type );
    binder.setMultiLine( multiLine );
    binder.setLabelProvider( labelProvider );
    return ( LabelBinding ) binder.bind( formModel, formPropertyPath, new HashMap() );
  }

  @Nullable
  private LabelProvider labelProvider;

  private boolean multiLine;

  public LabelBinder( @Nullable Class<T> requiredSourceClass ) {
    super( requiredSourceClass );
  }

  @Override
  protected JComponent createControl( Map context ) {
    if ( multiLine ) {
      JTextArea textArea = new JTextArea();
      textArea.setEditable( false );
      textArea.setLineWrap( true );
      textArea.setWrapStyleWord( true );
      textArea.setRows( 3 );
      textArea.setColumns( 40 );
      return new JScrollPane( textArea );
    } else {
      return new JLabel();
    }
  }

  public boolean isMultiLine() {
    return multiLine;
  }

  public void setMultiLine( boolean multiLine ) {
    this.multiLine = multiLine;
  }

  @Nullable
  public LabelProvider getLabelProvider() {
    return labelProvider;
  }

  public void setLabelProvider( @Nullable LabelProvider labelProvider ) {
    this.labelProvider = labelProvider;
  }

  @Override
  protected LabelBinding doBind( final JComponent control, FormModel formModel, String formPropertyPath, Map context ) {
    return new LabelBinding( formModel, formPropertyPath, control, getRequiredSourceClass(), labelProvider );
  }
}
