package com.cedarsoft.spring.rcp.binding;

import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.spring.rcp.renderer.DefaultLabelProvider;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanelFactory;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.binding.form.FormModel;
import org.springframework.richclient.form.binding.Binding;
import org.springframework.richclient.form.binding.support.AbstractBinder;
import org.springframework.richclient.selection.binding.support.LabelProvider;

import javax.swing.JComponent;
import javax.swing.JLabel;
import java.util.Map;

/**
 * Binds an single bean's property to an {@link ObjectTablePanel} shown in a dialog
 *
 * @param <T> the type
 */
public class ObjectTablePanelSingleBinder<T> extends AbstractBinder {
  @Nullable
  private NullableSupport nullableSupport;

  @NotNull
  private final LabelProvider labelProvider;
  @NotNull
  private final ObjectTablePanelFactory<T> objectTablePanelFactory;
  @NotNull
  private final ObservableObjectAccess<T> objectAccess;

  /**
   * Creates a new binder with the given factory
   *
   * @param objectAccess            the object access
   * @param objectTablePanelFactory the factory
   */
  public ObjectTablePanelSingleBinder( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory ) {
    this( objectAccess, new DefaultLabelProvider<T>() {
      @Override
      @NotNull
      protected String getLabelFor( @NotNull T item ) {
        return item.toString();
      }
    }, objectTablePanelFactory );
  }

  /**
   * Creates a new table binder
   *
   * @param objectAccess            the object access
   * @param labelProvider           the label provider used
   * @param objectTablePanelFactory the factory
   */
  public ObjectTablePanelSingleBinder( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull LabelProvider labelProvider, @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory ) {
    this( objectAccess, labelProvider, objectTablePanelFactory, null );
  }

  /**
   * Creates a new table binder
   *
   * @param objectAccess            the object access
   * @param labelProvider           the label provider
   * @param objectTablePanelFactory the factory
   * @param nullableSupport         the nullable support
   */
  public ObjectTablePanelSingleBinder( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull LabelProvider labelProvider, @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory, @Nullable NullableSupport nullableSupport ) {
    super( null );
    this.objectAccess = objectAccess;
    this.labelProvider = labelProvider;
    this.objectTablePanelFactory = objectTablePanelFactory;
    this.nullableSupport = nullableSupport;
  }

  @Override
  @NotNull
  protected JComponent createControl( Map context ) {
    return new JLabel( "dummy" );
  }

  @Override
  protected Binding doBind( @NotNull JComponent control, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath, Map context ) {
    boolean nullable;
    if ( nullableSupport == null ) {
      nullable = false;
    } else {
      nullable = nullableSupport.isNullable( formModel, formPropertyPath );
    }

    return new ObjectTablePanelSingleBinding<T>( objectAccess, labelProvider, objectTablePanelFactory, formModel, formPropertyPath, nullable );
  }

  @Nullable
  public NullableSupport getNullableSupport() {
    return nullableSupport;
  }

  public void setNullableSupport( @Nullable NullableSupport nullableSupport ) {
    this.nullableSupport = nullableSupport;
  }
}
