package com.cedarsoft.spring.rcp.commands;

import com.cedarsoft.utils.Condition;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.command.config.CommandConfigurer;

/**
 * A builder for a command
 *
 * @param <T> the type
 */
public class CommandBuilder<T> {
  /**
   * The condition
   */
  @Nullable
  private Condition condition;

  /**
   * The configurer
   */
  @Nullable
  private CommandConfigurer configurer;

  /**
   * The provider
   */
  @Nullable
  private CommandProvider<T> provider;

  @Nullable
  public Condition getCondition() {
    return condition;
  }

  public void setCondition( @Nullable Condition condition ) {
    this.condition = condition;
  }

  @Nullable
  public CommandConfigurer getConfigurer() {
    return configurer;
  }

  public void setConfigurer( @Nullable CommandConfigurer configurer ) {
    this.configurer = configurer;
  }

  @Nullable
  public CommandProvider<T> getProvider() {
    return provider;
  }

  public void setProvider( @Nullable CommandProvider<T> provider ) {
    this.provider = provider;
  }

  /**
   * Returns the command
   *
   * @param object the object used to create the command
   * @return the command (or null)
   */
  @Nullable
  public ActionCommand getCommand( @NotNull T object ) {
    if ( condition != null && !condition.isValid() ) {
      return null;
    }

    if ( provider != null ) {
      ActionCommand command = provider.getCommand( object );

      if ( configurer != null ) {
        configurer.configure( command );
      }

      return command;
    } else {
      return null;
    }
  }
}
