package com.cedarsoft.spring.rcp.commands;

import org.springframework.richclient.command.config.CommandConfigurer;
import org.springframework.richclient.command.AbstractCommand;
import org.jetbrains.annotations.NotNull;
import com.cedarsoft.PartTimeObjectAdd;

/**
 * Command configurer that enables/disables a command that is based on a {@link PartTimeObjectAdd}.
 */
public class PartTimeObjectAccessCommandConfigurer implements CommandConfigurer {
  private final PartTimeObjectAdd<?> partTimeObjectAdd;

  /**
   * Creates a new part time configurer
   *
   * @param partTimeObjectAdd the part time object add
   */
  public PartTimeObjectAccessCommandConfigurer( @NotNull PartTimeObjectAdd<?> partTimeObjectAdd ) {
    this.partTimeObjectAdd = partTimeObjectAdd;
  }

  @Override
  public AbstractCommand configure( @NotNull final AbstractCommand command ) {
    command.setEnabled( partTimeObjectAdd.canAdd() );

    partTimeObjectAdd.addPartTimeListener( new PartTimeObjectAdd.PartTimeListener() {
      @Override
      public void addNotAvailable() {
        command.setEnabled( false );
      }

      @Override
      public void addAvailable() {
        command.setEnabled( true );
      }
    } );

    return command;
  }
}
