package com.cedarsoft.spring.rcp.commands;

import com.cedarsoft.spring.rcp.selection.SelectionModel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * A command that is based on a selected object
 *
 * @param <T> the type
 */
public abstract class SelectionAwareCommand<T> extends ExtendedCommand {
  @NotNull
  protected final SelectionAwareCommandSupport<T> selectionAwareCommandSupport;

  /**
   * Creates a new command
   *
   * @param commandId      the command id
   * @param type           the type
   * @param selectionModel the selection model
   */
  protected SelectionAwareCommand( @NotNull @NonNls String commandId, @NotNull final Class<T> type, @NotNull SelectionModel selectionModel ) {
    this( commandId, type, selectionModel, null );
  }

  protected SelectionAwareCommand( @NotNull @NonNls String commandId, @NotNull final Class<T> type, @NotNull SelectionModel selectionModel, @Nullable SelectionAwareCommandSupport.Condition<T> condition ) {
    super( commandId );
    selectionAwareCommandSupport = new SelectionAwareCommandSupport<T>( type, selectionModel, this, condition );
  }

  /**
   * Returns the selected object
   *
   * @return the selected object
   */
  @NotNull
  public T getSelectedObject() {
    return selectionAwareCommandSupport.getSelectedObject();
  }

  public void updateEnabledState() {
    selectionAwareCommandSupport.updateEnabledState( this );
  }
}
