package com.cedarsoft.spring.rcp.conversion;

import org.jetbrains.annotations.NotNull;
import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.springframework.binding.convert.ConversionContext;
import org.springframework.binding.convert.support.ConversionServiceAwareConverter;

import java.util.Date;

/**
 * Converts a date to joda classes
 */
public class DateToJodaConverter extends ConversionServiceAwareConverter {
  @NotNull
  public static final LocalDate EARLIEST = new LocalDate( 1900, 1, 1 );
  @NotNull
  public static final LocalDate LATEST = new LocalDate( 2099, 12, 31 );

  @Override
  protected Object doConvert( Object source, Class targetClass, ConversionContext context ) throws Exception {
    Date date = ( Date ) source;

    //todo auto-Convert dates between year 0 and 100

    //Verify the dates --> else there might be a problem with joda...
    if ( date.before( EARLIEST.toDateTimeAtStartOfDay().toDate() ) ) {
      throw new IllegalArgumentException( "Date is too early: " + date );
    }
    if ( date.after( LATEST.toDateTimeAtStartOfDay().toDate() ) ) {
      throw new IllegalArgumentException( "Date is too late: " + date );
    }

    if ( targetClass.equals( LocalDate.class ) ) {
      return new LocalDate( date );
    } else if ( targetClass.equals( DateTime.class ) ) {
      return new DateTime( date );
    }

    throw new IllegalStateException( "??? " + targetClass );
  }

  @Override
  public Class[] getSourceClasses() {
    return new Class[]{Date.class};
  }

  @Override
  public Class[] getTargetClasses() {
    return new Class[]{LocalDate.class, DateTime.class, LocalDate.class};
  }
}
