package com.cedarsoft.spring.rcp.dialog;

import com.cedarsoft.CommitableObjectAccess;
import com.cedarsoft.spring.rcp.creation.CreatorModelFactory;
import com.cedarsoft.spring.rcp.forms.CreatorFormBackend;
import com.cedarsoft.spring.rcp.forms.FormFactory;
import com.cedarsoft.spring.rcp.forms.StaticFormBackend;
import com.cedarsoft.spring.rcp.id.IdStrategy;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Dialog factory that creates a dialog for a special business object.
 *
 * @param <T> the type of the object this factory creates dialogs for
 */
public class CreatorFormDialogFactory<T> implements FormDialogFactory<T> {
  @NotNull
  @NonNls
  private final String dialogId;
  @NotNull
  @NonNls
  private final String formId;

  @NotNull
  private final CreatorModelFactory<T> creatorModelFactory;
  @NotNull
  private final FormFactory<T> formFactory;

  /**
   * Creates a new dialog factory
   *
   * @param idStrategy          the id strategy
   * @param creatorModelFactory the creator model factory
   * @param formFactory         the form factory
   */
  public CreatorFormDialogFactory( @NotNull IdStrategy idStrategy, @NotNull CreatorModelFactory<T> creatorModelFactory, @NotNull FormFactory<T> formFactory ) {
    this( idStrategy.createDialogId(), idStrategy.createFormId(), creatorModelFactory, formFactory );
  }

  /**
   * Creates a  new factory
   *
   * @param dialogId            the id
   * @param creatorModelFactory the creator model factory
   * @param formFactory         the form factory
   * @param formId
   */
  public CreatorFormDialogFactory( @NotNull String dialogId, String formId, @NotNull CreatorModelFactory<T> creatorModelFactory, @NotNull FormFactory<T> formFactory ) {
    this.dialogId = dialogId;
    this.creatorModelFactory = creatorModelFactory;
    this.formFactory = formFactory;
    this.formId = formId;
  }

  @Override
  @NotNull
  public FormDialog<T> createDialog( @Nullable final T selectedObject, @NotNull final CommitableObjectAccess<T> objectAccess ) {
    if ( selectedObject != null ) {
      //Read only view!!!
      return new FormDialog<T>( dialogId, new StaticFormBackend<T>( formId, selectedObject, objectAccess ), formFactory );
    } else {
      return new FormDialog<T>( dialogId, new CreatorFormBackend<T>( creatorModelFactory.createCreatorModel( formId ), objectAccess ), formFactory );
    }
  }
}