package com.cedarsoft.spring.rcp.dialog;

import com.cedarsoft.spring.rcp.forms.ExtendedForm;
import com.cedarsoft.spring.rcp.forms.FormBackend;
import com.cedarsoft.spring.rcp.forms.FormFactory;
import com.cedarsoft.spring.rcp.forms.FormUi;
import com.cedarsoft.spring.rcp.forms.ReflectionFormFactory;
import com.cedarsoft.spring.rcp.forms.DefaultFormUi;
import com.cedarsoft.CanceledException;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.dialog.CloseAction;
import org.springframework.richclient.dialog.FormBackedDialogPage;
import org.springframework.richclient.dialog.TitledPageApplicationDialog;

/**
 * Base class for a properties dialog based on a form.
 *
 * @param <T> the type
 */
public class FormDialog<T> extends TitledPageApplicationDialog {
  @NotNull
  @NonNls
  protected final String id;
  @NotNull
  private final FormBackend backend;
  @NotNull
  private final FormUi<T> formUi;

  /**
   * Creates a new form dialog
   *
   * @param id       the id
   * @param backend  the model
   * @param formType the form type that is created using a {@link ReflectionFormFactory}
   */
  public FormDialog( @NonNls @NotNull String id, @NotNull FormBackend backend, @NotNull Class<? extends ExtendedForm<T>> formType ) {
    this( id, backend, ReflectionFormFactory.create( formType ) );
  }

  /**
   * Creates a new form dialog
   *
   * @param id          the id
   * @param backend     the model
   * @param formFactory the form factory
   */
  public FormDialog( @NonNls @NotNull String id, @NotNull FormBackend backend, @NotNull FormFactory<T> formFactory ) {
    this.id = id;
    this.backend = backend;
    formUi = new DefaultFormUi<T>( id, backend, formFactory );

    setCloseAction( CloseAction.DISPOSE );
  }

  @Override
  public void showDialog() throws CanceledException {
    setDialogPage( new FormBackedDialogPage( formUi.getForm() ) );
    super.showDialog();
  }

  @Override
  protected void onAboutToShow() {
    if ( backend.isCreatingNew() ) {
      setTitle( getMessage( id + ".new.title" ) );
    } else {
      setTitle( getMessage( id + ".title" ) );
    }
  }

  /**
   * @noinspection RefusedBequest
   */
  @Override
  protected void onCancel() {
    formUi.revert( new Runnable() {
      @Override
      public void run() {
        FormDialog.super.onCancel();
      }
    } );
  }

  @Override
  protected boolean onFinish() {
    formUi.commit();
    return true;
  }

  @NotNull
  @NonNls
  public String getId() {
    return id;
  }
}
