package com.cedarsoft.spring.rcp.events;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.richclient.application.event.LifecycleApplicationEvent;

/**
 * Abstract application lifecycle listener
 *
 * @param <T> the type
 */
public abstract class AbstractApplicationLifecycleListener<T> implements ApplicationListener {
  @NotNull
  private final Class<T> type;

  protected AbstractApplicationLifecycleListener( @NotNull Class<T> type ) {
    this.type = type;
  }

  @Override
  public void onApplicationEvent( ApplicationEvent event ) {
    if ( !( event instanceof LifecycleApplicationEvent ) ) {
      return;
    }

    LifecycleApplicationEvent lifecycleEvent = ( LifecycleApplicationEvent ) event;
    if ( !type.isAssignableFrom( lifecycleEvent.getObject().getClass() ) ) {
      return;
    }

    @NonNls
    String eventType = ( ( LifecycleApplicationEvent ) event ).getEventType();
    if ( LifecycleApplicationEvent.CREATED.equals( eventType ) || LifecycleApplicationEvent.DELETED.equals( eventType ) || LifecycleApplicationEvent.MODIFIED.equals( eventType ) ) {
      handle( this.type.cast( lifecycleEvent.getObject() ), lifecycleEvent );
    }
  }

  /**
   * Is called if an event of the configured type has been thrown
   *
   * @param object         the object (of the event)
   * @param lifecycleEvent the event itself
   */
  protected abstract void handle( @NotNull T object, @NotNull LifecycleApplicationEvent lifecycleEvent );
}
