package com.cedarsoft.spring.rcp.events;

import com.cedarsoft.lookup.Lookup;
import com.cedarsoft.lookup.Lookups;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.application.event.LifecycleApplicationEvent;

/**
 * This event is thrown when the selection has changed
 */
public class SelectionEvent extends LifecycleApplicationEvent {
  @NotNull
  public static final Object NULL = new Object();
  @NonNls
  public static final String SELECTION = "selected";

  @NotNull
  private final Lookup objects;
  @NotNull
  private final Type type;

  /**
   * Creates a new selection event
   *
   * @param type   the type
   * @param object the obbject
   */
  public SelectionEvent( @NotNull Type type, @NotNull Object object ) {
    this( type, Lookups.dynamicLookup( object ) );
  }

  /**
   * Creates a new selection event
   *
   * @param type    the type
   * @param objects the objects
   */
  public SelectionEvent( @NotNull Type type, @NotNull Lookup objects ) {
    super( SELECTION, objects );
    this.type = type;
    this.objects = objects;
  }

  @NotNull
  public Lookup getObjects() {
    return objects;
  }

  @NotNull
  public Type getType() {
    return type;
  }

  public enum Type {
    SELECT, UNSELECT
  }

}
