package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;

/**
 * The backend for a form.
 * The backend can be used to
 * <ul>
 * <li>commit the form {@link #commit()}
 * <li>check whether the form is based on a newly created bean {@link #isCreatingNew()}
 * <li>recieve the form model {@link #getFormModel()}
 * <li>ask whether the user shall be asked for commit confirmation {@link #shallAskForCommitConfirmation()}
 * </ul>
 */
public interface FormBackend {
  /**
   * Is called when the properties dialog is commited.
   */
  void commit();

  /**
   * Returns whether a new object is created
   *
   * @return whether a new object is created
   */
  boolean isCreatingNew();

  /**
   * Returns the form model
   *
   * @return the form model
   */
  @NotNull
  FormModel getFormModel();

  /**
   * Returns whether the user should be assed for commit confimations
   *
   * @return whether the user should be assed for commit confimations
   */
  boolean shallAskForCommitConfirmation();
}