package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import com.cedarsoft.CanceledException;

/**
 * Represents a (single!) form with all necessary objects to display/edit/commit that form
 *
 * @param <T> the type
 */
public interface FormUi<T> {
  /**
   * Returns the form
   *
   * @return the form
   *
   * @throws CanceledException
   */
  @NotNull
  ExtendedForm<? extends T> getForm() throws CanceledException;

  /**
   * Returns the form object
   *
   * @return the form object
   */
  @NotNull
  T getFormObject();

  /**
   * Reverts the form
   */
  void revert();

  /**
   * Reverts the form and calls the callback.
   * If the form is dirty, a confirmation dialog will be shown. The callback is only called
   * if the user confirms the revert.
   *
   * @param callback the callback.
   */
  void revert( @Nullable Runnable callback );

  /**
   * Commits the form
   */
  void commit();

  /**
   * Commits the form
   *
   * @param shallAskForConfirmation whether should ask for confirmation
   * @throws CanceledException
   */
  void commit( boolean shallAskForConfirmation ) throws CanceledException;
}