package com.cedarsoft.spring.rcp.forms;

import com.cedarsoft.CommitableObjectAccess;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Form backend based on a commitable object access
 *
 * @param <T> the type
 */
public abstract class ObjectAccessBasedFormBackend<T> extends AbstractFormBackend<T> {
  @NotNull
  private final CommitableObjectAccess<? super T> objectAccess;

  /**
   * Creates a new form backend
   *
   * @param formId       the form id
   * @param objectAccess the object access
   * @param object       the selected object (or null if a new object will be created).
   */
  protected ObjectAccessBasedFormBackend( @NotNull @NonNls String formId, @NotNull CommitableObjectAccess<? super T> objectAccess, @Nullable T object ) {
    super( formId, object, object == null );
    this.objectAccess = objectAccess;
  }

  /**
   * Creates a new form backend
   *
   * @param objectAccess     the object access
   * @param object           the object (may be null if a new object is created)
   * @param formModelFactory the form model factory
   */
  protected ObjectAccessBasedFormBackend( @NotNull CommitableObjectAccess<? super T> objectAccess, @Nullable T object, @NotNull FormModelFactory<T> formModelFactory ) {
    super( object, object == null, formModelFactory );
    this.objectAccess = objectAccess;
  }

  @Override
  public void addObject( @NotNull T addedObject ) {
    objectAccess.add( addedObject );
  }

  @Override
  public void commitObject( @NotNull T commitedObject ) {
    objectAccess.commit( commitedObject );
  }
}
