package com.cedarsoft.spring.rcp.forms;

import com.cedarsoft.ObjectCommit;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;

/**
 * Static form backend that commits the objects using the given object commit
 *
 * @param <T> the type
 */
public class StaticFormBackend<T> extends AbstractStaticFormBackend<T> {
  @NotNull
  private final ObjectCommit<T> objectCommit;

  /**
   * Creates a new for form backend
   *
   * @param formModel    the form model
   * @param objectCommit the object commit that is used to commit the changes
   */
  public StaticFormBackend( @NotNull FormModel formModel, @NotNull ObjectCommit<T> objectCommit ) {
    super( formModel );
    assert formModel.getId() != null;
    this.objectCommit = objectCommit;
  }

  /**
   * Creates a new form backend
   *
   * @param formId       the form id
   * @param object       the object the form is created for
   * @param objectCommit the object commit that is used to commit the changes
   */
  public StaticFormBackend( @NotNull @NonNls String formId, @NotNull T object, @NotNull ObjectCommit<T> objectCommit ) {
    super( formId, object );
    this.objectCommit = objectCommit;
  }

  @Override
  public void commit() {
    objectCommit.commit( getFormObject() );
  }
}
