package com.cedarsoft.spring.rcp.hierarchy;

import com.cedarsoft.spring.rcp.events.DetailsTreeSelectionEvent;
import com.cedarsoft.utils.Renderer;
import com.cedarsoft.utils.TypeRegistry;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Creates hierarchy models.
 * Those models are independent from each other (different selection) and can be used within the editors
 */
public class HierarchyModelFactory implements ApplicationListener {
  @NotNull
  private final Renderer<Object, Object> fallbackRenderer;
  @NotNull
  private final TypeRegistry<HierarchyLayer<?, ?>> hierarchyLayerRegistry;

  public HierarchyModelFactory( @NotNull Renderer<Object, Object> fallbackRenderer, @NotNull TypeRegistry<HierarchyLayer<?, ?>> hierarchyLayerRegistry ) {
    this.fallbackRenderer = fallbackRenderer;
    this.hierarchyLayerRegistry = hierarchyLayerRegistry;
  }

  @NotNull
  public HierachyTreeModel createHierarchyTreeModel( @NotNull Object root ) {
    return new HierachyTreeModel( root, fallbackRenderer, hierarchyLayerRegistry );
  }

  @NotNull
  public HierarchyDetailsModel createDetailsModel( @NotNull Object root ) {
    HierarchyDetailsModel model = new HierarchyDetailsModel( hierarchyLayerRegistry );
    model.select( new DetailsTreeSelectionEvent( DetailsTreeSelectionEvent.Type.SELECTED, root ) );
    detailModels.add( new WeakReference<HierarchyDetailsModel>( model ) );
    return model;
  }

  @NotNull
  private final List<WeakReference<HierarchyDetailsModel>> detailModels = new ArrayList<WeakReference<HierarchyDetailsModel>>();

  @Override
  public void onApplicationEvent( @NotNull ApplicationEvent event ) {
    for ( Iterator<WeakReference<HierarchyDetailsModel>> it = detailModels.iterator(); it.hasNext(); ) {
      WeakReference<HierarchyDetailsModel> reference = it.next();
      HierarchyDetailsModel model = reference.get();
      if ( model == null ) {
        it.remove();
      } else {
        model.onApplicationEvent( event );
      }
    }
  }
}
