package com.cedarsoft.spring.rcp.hierarchy;

import com.cedarsoft.utils.Renderer;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import java.awt.Component;

/**
 *
 */
public abstract class ManagedTreeCellRenderer extends DefaultTreeCellRenderer {
  @NotNull
  private final Renderer<Object, Object> fallbackRenderer;

  protected ManagedTreeCellRenderer( @NotNull Renderer<Object, Object> fallbackRenderer ) {
    this.fallbackRenderer = fallbackRenderer;
  }

  @Override
  public Component getTreeCellRendererComponent( @NotNull JTree tree, @Nullable Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus ) {
    super.getTreeCellRendererComponent( tree, value, sel, expanded, leaf, row, hasFocus );
    if ( value != null ) {
      Renderer<Object, Object> renderer = getRendererSafe( value );
      setText( renderer.render( value, null ) );
    }
    return this;
  }

  /**
   * Helper method that returns a renderer for the given value
   *
   * @param value the value
   * @return the renderer
   */
  @NotNull
  private Renderer<Object, Object> getRendererSafe( @NotNull Object value ) {
    Renderer<Object, Object> renderer;
    try {
      renderer = getRenderer( value );
      if ( renderer == null ) {
        renderer = fallbackRenderer;
      }
    } catch ( IllegalArgumentException ignore ) {
      renderer = fallbackRenderer;
    }
    return renderer;
  }

  /**
   * Returns the renderer for the given object
   *
   * @param value the object
   * @return the renderer
   * @throws IllegalArgumentException
   */
  @Nullable
  protected abstract Renderer<Object, Object> getRenderer( @NotNull Object value ) throws IllegalArgumentException;
}
