package com.cedarsoft.spring.rcp.prefetch;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Collections;

/**
 * This implementation is based on reflection and calls all getters
 */
public class ReflectionObjectPrefetcher extends AbstractObjectPrefetcher<Object> {
  @NotNull
  private static final Log log = LogFactory.getLog( ReflectionObjectPrefetcher.class );

  /**
   * Creates a new reflection prefetcher
   */
  public ReflectionObjectPrefetcher() {
    super( Object.class );
  }

  @Override
  @NotNull
  public Collection<? extends Object> prefetch( @NotNull Object object ) {
    this.prefetchGetters( object );
    return Collections.emptyList();
  }

  protected void prefetchGetters( @NotNull Object object ) {
    //Call all getters
    for ( Method method : object.getClass().getMethods() ) {
      if ( !method.getName().startsWith( "get" ) ) {
        continue;
      }
      if ( method.getParameterTypes().length > 0 ) {
        continue;
      }

      try {
        method.invoke( object );
      } catch ( InvocationTargetException e ) {
        log.info( "---> prefetching failed " + e.getTargetException() );
      } catch ( Exception e ) {
        log.info( "---> prefetching failed " + e );
      }
    }
  }
}
