package com.cedarsoft.spring.rcp.table.renderer;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.Component;

/**
 * Abstract base class for custom table cell renderers.
 * Contains support for alignment to the right
 * @param <T> the type
 */
public abstract class AbstractTableCellRenderer<T> extends DefaultTableCellRenderer {
  @NotNull
  private final Class<T> type;
  private final boolean alignRight;

  protected AbstractTableCellRenderer( @NotNull Class<T> type ) {
    this( type, false );
  }

  protected AbstractTableCellRenderer( @NotNull Class<T> type, boolean alignRight ) {
    this.type = type;
    this.alignRight = alignRight;
  }

  @Override
  public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column ) {
    super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );
    if ( value != null ) {
      if ( type.isAssignableFrom( value.getClass() ) ) {
        if ( alignRight ) {
          setHorizontalAlignment( RIGHT );
        }
        String text = format( type.cast( value ) );
        setText( text );
        String toolTipText = getToolTipText( type.cast( value ) );
        if ( toolTipText != null ) {
          setToolTipText( toolTipText );
        } else {
          setToolTipText( text );
        }
      }
    }
    return this;
  }

  @Nullable
  protected String getToolTipText( @NotNull T object ) {
    return null;
  }

  @NotNull
  protected abstract String format( @NotNull T object );
}
