package com.cedarsoft.spring.rcp.tbpanel;

import com.cedarsoft.spring.SpringSupport;
import com.cedarsoft.spring.rcp.table.ExtendedObjectTable;
import com.cedarsoft.spring.rcp.table.ObjectTable;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationContext;
import org.springframework.richclient.application.Application;
import org.springframework.richclient.application.ApplicationServices;
import org.springframework.richclient.application.ApplicationServicesLocator;
import org.springframework.richclient.core.UIConstants;
import org.springframework.richclient.factory.ComponentFactory;

import javax.swing.JPanel;
import java.awt.BorderLayout;

/**
 * Abstract base class for filter factories
 *
 * @param <T> the type
 */
public abstract class AbstractFilterFactory<T> implements FilterFactory<T> {
  @NonNls
  protected static final String KEY_LABEL = "label";
  @NonNls
  protected static final String KEY_FILTER = "filter";
  @NonNls
  protected static final String KEY_FILTER_LABEL = KEY_FILTER + '.' + KEY_LABEL;

  @Override
  @NotNull
  public JPanel createFilterPanel( @NotNull ObjectTable<T> objectTable ) {
    JPanel filterPanel = new JPanel( new BorderLayout( UIConstants.ONE_SPACE, UIConstants.ONE_SPACE ) );
    fillFilterPanel( filterPanel, objectTable );
    return filterPanel;
  }

  /**
   * Is called to fill the filter panel with the filter components
   *
   * @param filterPanel the filter panel that shall be filled
   * @param objectTable the object table
   */
  protected abstract void fillFilterPanel( @NotNull JPanel filterPanel, @NotNull ObjectTable<T> objectTable );

  @NotNull
  protected Application getApplication() {
    return Application.instance();
  }

  @NotNull
  protected ApplicationServices getApplicationServices() {
    return ApplicationServicesLocator.services();
  }

  @NotNull
  protected <T> T getService( Class<T> serviceType ) {
    return serviceType.cast( getApplicationServices().getService( serviceType ) );
  }

  @NotNull
  protected ApplicationContext getApplicationContext() {
    return getApplication().getApplicationContext();
  }

  @NotNull
  protected ComponentFactory getComponentFactory() {
    return getService( ComponentFactory.class );
  }

  @NotNull
  protected String getMessage( @NotNull @NonNls String messageCode ) {
    return SpringSupport.INSTANCE.getMessage( messageCode );
  }
}
