package com.cedarsoft.spring.rcp.tbpanel;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.FilterList;
import ca.odell.glazedlists.GlazedLists;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.TextFilterator;
import ca.odell.glazedlists.swing.TextComponentMatcherEditor;
import com.cedarsoft.spring.rcp.table.ExtendedObjectTable;
import com.cedarsoft.spring.rcp.table.FinalEventListFactory;
import com.cedarsoft.spring.rcp.table.ObjectTable;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.awt.BorderLayout;

/**
 * Creates a filter that is based on one text field
 *
 * @param <T> the type
 */
public class SingleFieldFilterFactory<T> extends AbstractFilterFactory<T> {
  @NotNull
  @NonNls
  protected final String[] filteredProperties;
  @NotNull
  @NonNls
  private final String objectId;
  @NotNull
  @NonNls
  private static final String KEY_SUFFIX_FILTER_CAPTION = ".filter.caption";

  /**
   * Creates a new filter factory
   *
   * @param objectId           the object id
   * @param filteredProperties the filtered properties
   */
  public SingleFieldFilterFactory( @NotNull @NonNls String objectId, @NonNls String... filteredProperties ) {
    this.objectId = objectId;
    this.filteredProperties = filteredProperties.clone();
  }

  @Override
  protected void fillFilterPanel( @NotNull JPanel filterPanel, @NotNull ObjectTable<T> objectTable ) {
    final JTextField filterField = getComponentFactory().createTextField();

    JLabel filterLabel = getComponentFactory().createLabel( createMessageKeys( objectId, filteredProperties ) );
    filterPanel.add( filterLabel, BorderLayout.WEST );
    filterField.setToolTipText( getMessage( objectId + KEY_SUFFIX_FILTER_CAPTION ) );
    filterPanel.add( filterField, BorderLayout.CENTER );

    configureFiltering( objectTable, filterField );
  }

  @NotNull
  @NonNls
  private static String[] createMessageKeys( @NotNull @NonNls String objectId, @NotNull @NonNls String[] filteredProperties ) {
    return new String[]{objectId + '.' + KEY_FILTER_LABEL,
                        filteredProperties[0] + '.' + KEY_FILTER_LABEL,
                        filteredProperties[0] + '.' + KEY_LABEL,
                        filteredProperties[0] + '.' + KEY_LABEL
    };
  }

  public void configureFiltering( @NotNull ObjectTable<T> objectTable, @NotNull final JTextField filterField ) {
    // Construct and install our filtering list. This filter will allow the user
    // to simply type data into the txtFilter (JTextField). With the configuration
    // setup below, the text entered by the user will be matched against the values
    // in the lastName and address.address1 properties of the contacts in the table.
    // The GlazedLists filtered lists is used to accomplish this.

    // Install the fully constructed (layered) list into the table
    objectTable.setFinalEventListFactory( new FinalEventListFactory<T>() {
      @NotNull
      @Override
      public EventList<T> create( @NotNull SortedList<T> baseEventList ) {
        TextFilterator<T> filterator = GlazedLists.<T>textFilterator( filteredProperties );
        return new FilterList<T>( baseEventList, new TextComponentMatcherEditor<T>( filterField, filterator ) );
      }
    } );
  }
}
