package com.cedarsoft.spring.rcp.tree;

import com.cedarsoft.spring.rcp.ComponentProvidersSupport;
import com.cedarsoft.spring.rcp.views.ExtendedView;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.core.UIConstants;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.awt.BorderLayout;

/**
 * A special view that contains a JTree
 */
public abstract class TreeView extends ExtendedView {
  private JTree tree;

  @NotNull
  public JTree getTree() {
    if ( tree == null ) {
      throw new IllegalStateException( "Tree has not been created yet" );
    }
    return tree;
  }

  @Override
  protected JComponent createControl() {
    this.tree = createTree();

    tree.getSelectionModel().addTreeSelectionListener( new TreeSelectionListener() {
      @Override
      public void valueChanged( TreeSelectionEvent e ) {
        updateStatusBar();
      }
    } );

    JScrollPane treePane = getComponentFactory().createScrollPane( tree );

    JPanel panel = getComponentFactory().createPanel( new BorderLayout() );
    panel.add( treePane, BorderLayout.CENTER );

    if ( headComponentSupport.providersSet() ) {
      panel.add( headComponentSupport.create(), BorderLayout.NORTH );
    }

    panel.setBorder( BorderFactory.createEmptyBorder( UIConstants.ONE_SPACE, 0, 0, 0 ) );
    return panel;
  }

  @NotNull
  private final ComponentProvidersSupport headComponentSupport = new ComponentProvidersSupport();

  @NotNull
  public ComponentProvidersSupport getHeadComponentSupport() {
    return headComponentSupport;
  }

  /**
   * Creates the tree for this view
   *
   * @return the created tree
   */
  @NotNull
  protected abstract JTree createTree();
}
