package com.cedarsoft.spring.rcp.tree.layered;

import com.cedarsoft.spring.rcp.tree.TreeUtils;
import org.jetbrains.annotations.NotNull;

import javax.swing.tree.TreePath;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Mouse click listener based on a layers configuration
 */
public class LayeredDoubleClickListener extends MouseAdapter {
  @NotNull
  private final LayersConfiguration layersConfiguration;

  public LayeredDoubleClickListener( @NotNull LayersConfiguration layersConfiguration ) {
    this.layersConfiguration = layersConfiguration;
  }

  @Override
  public void mousePressed( MouseEvent e ) {
    if ( e.getClickCount() != 2 ) {
      return;
    }

    TreePath clickedPath = TreeUtils.getClickedPath( e );
    if ( clickedPath == null ) {
      return;
    }

    Object clickedNode = clickedPath.getLastPathComponent();
    Object clickedUserObject = TreeUtils.getUserObject( clickedNode );

    Object parentNode = clickedPath.getParentPath().getLastPathComponent();
    Object parent = TreeUtils.getUserObject( parentNode );

    TreeLayer<Object, Object> treeLayer = ( TreeLayer<Object, Object> ) layersConfiguration.getLayer( parent.getClass() );
    treeLayer.handleDoubleClick( parent, clickedUserObject, e );
  }
}
