package com.cedarsoft.spring.rcp.tree.layered;

import com.cedarsoft.spring.rcp.tree.TreeUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.util.PopupMenuMouseListener;

import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;
import java.awt.event.MouseEvent;

/**
 * Mouse click listener based on a layers configuration
 */
public class LayeredPopupListener extends PopupMenuMouseListener {
  @NotNull
  private final LayersConfiguration layersConfiguration;

  public LayeredPopupListener( @NotNull LayersConfiguration layersConfiguration ) {
    this.layersConfiguration = layersConfiguration;
  }

  @Override
  @Nullable
  protected JPopupMenu getPopupMenu( MouseEvent e ) {
    TreePath clickedPath = TreeUtils.getClickedPath( e );
    if ( clickedPath == null ) {
      return null;
    }

    Object clickedNode = clickedPath.getLastPathComponent();
    Object clickedUserObject = TreeUtils.getUserObject( clickedNode );

    Object parentNode = clickedPath.getParentPath().getLastPathComponent();
    Object parent = TreeUtils.getUserObject( parentNode );

    TreeLayer<Object, Object> treeLayer = ( TreeLayer<Object, Object> ) layersConfiguration.getLayer( parent.getClass() );
    return treeLayer.createPopupMenu( parent, clickedUserObject, e );
  }
}