package com.cedarsoft.spring.rcp.tree.layered.config;

import com.cedarsoft.spring.SpringSupport;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.application.config.ApplicationWindowAware;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.command.CommandGroup;
import org.springframework.richclient.util.RcpSupport;

import javax.swing.JTree;
import java.util.List;

/**
 * Configures a popup
 */
public abstract class PopupConfigurer<P, C> {
  public void configure( @NotNull CommandGroup popupCommandGroup, @NotNull P parent, @NotNull C selectedChild, @NotNull JTree source ) {
    List<? extends ActionCommand> commands = getCommands( parent, selectedChild, source );
    if ( commands == null ) {
      return;
    }

    for ( ActionCommand command : commands ) {
      RcpSupport.configure( command );
      if ( command instanceof ApplicationWindowAware ) {
        ( ( ApplicationWindowAware ) command ).setApplicationWindow( SpringSupport.INSTANCE.getActiveApplicationWindow() );
      }
      popupCommandGroup.add( command );
    }
  }

  /**
   * Returns the commands
   *
   * @param parent        the parent
   * @param selectedChild the selected child
   * @param source the source
   * @return the list of commands
   */
  @Nullable
  protected abstract List<? extends ActionCommand> getCommands( @NotNull P parent, @NotNull C selectedChild, @NotNull JTree source );
}
