package com.cedarsoft.spring.rcp.view;

import com.cedarsoft.spring.rcp.views.ExtendedView;
import com.jidesoft.docking.DockContext;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.richclient.application.support.DefaultViewDescriptor;
import org.springframework.richclient.command.config.CommandButtonLabelInfo;
import org.springframework.richclient.command.config.CommandLabelConfigurable;
import org.springframework.richclient.core.DescriptionConfigurable;
import org.springframework.richclient.core.TitleConfigurable;
import org.springframework.richclient.image.config.ImageConfigurable;

import java.awt.Image;
import java.awt.Rectangle;
import java.util.Map;

/**
 * @param <T> the view descriptor type
 */
public abstract class AbstractViewDescriptorFactory<T extends DefaultViewDescriptor> implements FactoryBean, BeanNameAware, CommandLabelConfigurable, TitleConfigurable, DescriptionConfigurable, ImageConfigurable {
  @NotNull
  protected final ViewConfiguration viewConfiguration;
  protected String description;
  protected Image image;
  protected CommandButtonLabelInfo labelInfo;
  @NonNls
  protected String name;
  @NonNls
  protected String title;
  protected String caption;
  protected boolean alwaysVisible;
  protected int initMode = DockContext.STATE_FRAMEDOCKED;
  protected int initSide = DockContext.DOCK_SIDE_EAST;
  protected int initIndex;
  protected boolean floatOnShow;
  protected Rectangle floatBounds;
  protected Boolean autohidable;
  protected Boolean floatable;
  protected Boolean dockable;
  protected Boolean hidable;
  protected Boolean showGripper;
  protected Boolean showTitleBar;
  protected Boolean sideDockAllowed;
  protected Boolean slidingAutohide;
  protected Boolean tabDockAllowed;
  protected Boolean maximizable;
  protected Boolean rearrangable;

  protected AbstractViewDescriptorFactory( @NotNull Class<? extends ExtendedView> viewClass, @NotNull @NonNls Map<String, Object> viewProperties ) {
    this( new DefaultViewConfiguration( viewClass, viewProperties ) );
  }

  protected AbstractViewDescriptorFactory( @NotNull ViewConfiguration viewConfiguration ) {
    this.viewConfiguration = viewConfiguration;
  }

  @Override
  public Class getObjectType() {
    return DefaultViewDescriptor.class;
  }

  @Override
  public Object getObject() throws Exception {
    T descriptor = createPlainDescriptor();
    assert name != null;
    descriptor.setId( name );

    descriptor.setLabelInfo( labelInfo );
    descriptor.setTitle( title );
    descriptor.setCaption( caption );
    descriptor.setDescription( description );
    descriptor.setImage( image );


    descriptor.setViewClass( viewConfiguration.getViewClass() );
    descriptor.setViewProperties( viewConfiguration.getViewProperties() );

    addCustomInitialization( descriptor );

    return descriptor;
  }

  /**
   * Just creates an *empty* descriptor
   *
   * @return an empty descriptor
   */
  @NotNull
  protected abstract T createPlainDescriptor();

  /**
   * Adds the custom initializion for the descriptor
   *
   * @param descriptor the descriptor that has been filled
   */
  protected abstract void addCustomInitialization( @NotNull T descriptor );

  @Override
  public void setBeanName( @NotNull @NonNls String name ) {
    this.name = name;
  }

  @Override
  public void setTitle( String title ) {
    this.title = title;
  }

  @Override
  public void setLabelInfo( CommandButtonLabelInfo labelInfo ) {
    this.labelInfo = labelInfo;
  }

  @Override
  public void setDescription( String longDescription ) {
    description = longDescription;
  }

  @Override
  public void setImage( Image image ) {
    this.image = image;
  }

  @Override
  public void setCaption( String shortDescription ) {
    this.caption = shortDescription;
  }

  @Override
  public boolean isSingleton() {
    return true;
  }

  public boolean isAlwaysVisible() {
    return alwaysVisible;
  }

  public void setAlwaysVisible( boolean alwaysVisible ) {
    this.alwaysVisible = alwaysVisible;
  }

  public int getInitMode() {
    return initMode;
  }

  public void setInitMode( int initMode ) {
    this.initMode = initMode;
  }

  public int getInitSide() {
    return initSide;
  }

  public void setInitSide( int initSide ) {
    this.initSide = initSide;
  }

  public int getInitIndex() {
    return initIndex;
  }

  public void setInitIndex( int initIndex ) {
    this.initIndex = initIndex;
  }

  public boolean isFloatOnShow() {
    return floatOnShow;
  }

  public void setFloatOnShow( boolean floatOnShow ) {
    this.floatOnShow = floatOnShow;
  }

  public Rectangle getFloatBounds() {
    return floatBounds;
  }

  public void setFloatBounds( Rectangle floatBounds ) {
    this.floatBounds = floatBounds;
  }

  public Boolean getAutohidable() {
    return autohidable;
  }

  public void setAutohidable( Boolean autohidable ) {
    this.autohidable = autohidable;
  }

  public Boolean getFloatable() {
    return floatable;
  }

  public void setFloatable( Boolean floatable ) {
    this.floatable = floatable;
  }

  public Boolean getDockable() {
    return dockable;
  }

  public void setDockable( Boolean dockable ) {
    this.dockable = dockable;
  }

  public Boolean getHidable() {
    return hidable;
  }

  public void setHidable( Boolean hidable ) {
    this.hidable = hidable;
  }

  public Boolean getShowGripper() {
    return showGripper;
  }

  public void setShowGripper( Boolean showGripper ) {
    this.showGripper = showGripper;
  }

  public Boolean getShowTitleBar() {
    return showTitleBar;
  }

  public void setShowTitleBar( Boolean showTitleBar ) {
    this.showTitleBar = showTitleBar;
  }

  public Boolean getSideDockAllowed() {
    return sideDockAllowed;
  }

  public void setSideDockAllowed( Boolean sideDockAllowed ) {
    this.sideDockAllowed = sideDockAllowed;
  }

  public Boolean getSlidingAutohide() {
    return slidingAutohide;
  }

  public void setSlidingAutohide( Boolean slidingAutohide ) {
    this.slidingAutohide = slidingAutohide;
  }

  public Boolean getTabDockAllowed() {
    return tabDockAllowed;
  }

  public void setTabDockAllowed( Boolean tabDockAllowed ) {
    this.tabDockAllowed = tabDockAllowed;
  }

  public Boolean getMaximizable() {
    return maximizable;
  }

  public void setMaximizable( Boolean maximizable ) {
    this.maximizable = maximizable;
  }

  public Boolean getRearrangable() {
    return rearrangable;
  }

  public void setRearrangable( Boolean rearrangable ) {
    this.rearrangable = rearrangable;
  }
}
