package com.cedarsoft.spring.rcp.view;

import com.cedarsoft.spring.rcp.selection.NoElementSelectedException;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanel;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanelFactory;
import com.cedarsoft.spring.rcp.views.ExtendedView;
import com.jgoodies.forms.factories.Borders;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.lookup.Lookup;
import com.cedarsoft.lookup.Lookups;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.core.UIConstants;

import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.BorderLayout;

/**
 * Abstract base class for table based views.
 * There only needs to be the {@link #objectTablePanelFactory} set.
 *
 * @param <T> the type
 */
public class ObjectTableView<T> extends ExtendedView {
  @NotNull
  @NonNls
  public static final String PROPERTY_TABLE_BASED_PANEL_FACTORY = "objectTablePanelFactory";
  @NotNull
  @NonNls
  public static final String PROPERTY_OBJECT_ACCESS = "objectAccess";

  @Override
  public void componentFocusGained() {
    super.componentFocusGained();
    getObjectTablePanel().updateContext( getContext() );
  }

  @Override
  @NotNull
  protected Lookup getSelectedObjects() {
    try {
      return Lookups.dynamicLookup( getObjectTablePanel().getObjectTable().getSelectedObject() );
    } catch ( NoElementSelectedException ignore ) {
      return Lookups.emtyLookup();
    }
  }

  @NotNull
  private final JPanel contentPanel = new JPanel( new BorderLayout( UIConstants.ONE_SPACE, UIConstants.ONE_SPACE ) );

  @Override
  protected JComponent createControl() {
    ObjectTablePanel<T> panel = getObjectTablePanel();
    panel.updateContext( getContext() );

    contentPanel.add( panel, BorderLayout.CENTER );
    contentPanel.setBorder( Borders.DLU2_BORDER );

    //todo add head/tail component(?)
    return contentPanel;
  }

  private ObjectTablePanel<T> objectTablePanel;

  @NotNull
  public ObjectTablePanel<T> getObjectTablePanel() {
    if ( objectTablePanel == null ) {
      objectTablePanel = createObjectTablePanel();
    }
    return objectTablePanel;
  }

  private ObjectTablePanelFactory<T> objectTablePanelFactory;

  public void setObjectTablePanelFactory( @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory ) {
    this.objectTablePanelFactory = objectTablePanelFactory;
  }

  private ObservableObjectAccess<T> objectAccess;

  @NotNull
  public ObservableObjectAccess<T> getObjectAccess() {
    if ( objectAccess == null ) {
      throw new IllegalStateException( "Object access not initialized properly" );
    }
    return objectAccess;
  }

  public void setObjectAccess( @NotNull ObservableObjectAccess<T> objectAccess ) {
    this.objectAccess = objectAccess;
  }

  @NotNull
  protected ObjectTablePanel<T> createObjectTablePanel() {
    if ( objectTablePanelFactory == null ) {
      throw new IllegalStateException( "objectTablePanelFactory not initalized properly" );
    }
    return objectTablePanelFactory.createPanel( getObjectAccess() );
  }
}
