package com.cedarsoft.spring.rcp.views;

import com.cedarsoft.spring.rcp.events.ApplicationAlertEvent;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.richclient.application.PageComponentContext;
import org.springframework.richclient.application.docking.jide.view.JideAbstractView;
import org.springframework.richclient.command.support.AbstractActionCommandExecutor;
import org.springframework.richclient.command.support.GlobalCommandIds;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.io.PrintWriter;
import java.io.StringWriter;

/**
 *
 */
public class ErrorMessagesView extends JideAbstractView implements ApplicationListener {
  @NotNull
  private final CopyCommandExecutor copyExecutor = new CopyCommandExecutor();
  private JScrollPane messageScrollPane;
  private JTextPane messageTextPane;

  @Override
  public void onApplicationEvent( ApplicationEvent event ) {
    if ( event instanceof ApplicationAlertEvent ) {
      ApplicationAlertEvent alertEvent = ( ApplicationAlertEvent ) event;
      if ( alertEvent.isErrorEvent() ) {
        if ( alertEvent.getSource() != null && alertEvent.getSource() instanceof Throwable ) {
          Throwable throwable = ( Throwable ) alertEvent.getSource();
          setErrorMessage( getStackTraceString( throwable ) );
        } else {
          setErrorMessage( alertEvent.getMessage() );
        }
      }
    }
  }

  @Override
  protected JComponent createControl() {
    if ( messageTextPane == null ) {
      messageTextPane = new JTextPane();
      messageTextPane.setEditable( false );
      messageScrollPane = getComponentFactory().createScrollPane( messageTextPane );
    }
    return messageScrollPane;
  }

  @Override
  protected void registerLocalCommandExecutors( PageComponentContext context ) {
    super.registerLocalCommandExecutors( context );
    context.register( GlobalCommandIds.COPY, copyExecutor );
  }

  @NotNull
  @NonNls
  private static String getStackTraceString( @NotNull Throwable throwable ) {
    StringWriter out = new StringWriter();
    //noinspection IOResourceOpenedButNotSafelyClosed
    throwable.printStackTrace( new PrintWriter( out ) );
    return out.toString();
  }

  private void setErrorMessage( @NotNull final String errorMessage ) {
    messageTextPane.setText( errorMessage );
  }

  private class CopyCommandExecutor extends AbstractActionCommandExecutor {
    @Override
    public void execute() {
      Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
      clipboard.setContents( new StringSelection( messageTextPane.getText() ), null );
    }
  }
}
