package com.cedarsoft.utils.springrcp.beanlist;

import com.cedarsoft.utils.springrcp.beanlist.command.DefaultBeanListCommandFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.application.PageComponentContext;
import org.springframework.richclient.command.CommandGroup;
import org.springframework.richclient.command.support.GlobalCommandIds;

import javax.swing.JPopupMenu;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * A BeanListView is a special view that can be used to administrate beans of a special type.
 * Therefore it is necessary to configure the following properties:
 * <ul>
 * <li>String beanName (the name of the bean, used for i18n)
 * <li>BeanDialog beanDialog (containing the form)
 * <li>BeanManager beanManager (manages the bean)
 * <li>TableModel tableModel (the beanlist model itself)
 * </ul>
 * <p/>
 * <p/>
 * <p/>
 * T is the type of the beans that are shown within this view.
 * <p/>
 * Date: 18.08.2006<br>
 * Time: 15:43:12<br>
 *
 * @author <a href="http://johannes-schneider.info">Johannes Schneider</a> -
 *         <a href="http://www.xore.de">Xore Systems</a>
 */
@Deprecated
public class BeanListView<T> extends AbstractBeanListView<T> {
  private BeanListCommandFactory commandFactory;
  private TableModel tableModel;

  private Map<Class<?>, TableCellRenderer> defaultTableCellRenderers = new HashMap<Class<?>, TableCellRenderer>();

  private Class<T> beanType;
  private BeanManager<T> beanManager;

  private BeanPopupMenuCreator<T> beanPopupMenuCreator;

  /**
   * This method is called as last step of initialization
   */
  @Override
  protected void finalizeInitialization() {
  }

  @Override
  protected void initializeCommands() {
    getCommandFactory().initialize();
  }

  @Override
  @NotNull
  protected CommandGroup createDefaultCommandGroup() {
    CommandGroup commandGroup = getWindowCommandManager().createCommandGroup( getBeanType().getName() + ".commandGroup", new Object[0] );
    commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getNewCommand() ) );
    commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getPropertiesCommand() ) );
    commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getDeleteCommand() ) );
    return commandGroup;
  }

  @Nullable
  @Override
  protected JPopupMenu showTablePopup() {
    return getBeanPopupMenuCreator().createMenu( this );
  }

  @Override
  protected void installCellRenderers() {
    super.installCellRenderers();
    //the configured cell renderers
    for ( Map.Entry<Class<?>, TableCellRenderer> entry : defaultTableCellRenderers.entrySet() ) {
      getTable().setDefaultRenderer( entry.getKey(), entry.getValue() );
    }
  }

  /**
   * Open the details for the selected bean.
   * This method is called, when the user double clicked the bean
   *
   * @param bean the selected bean
   */
  @Override
  protected void openBean( @NotNull T bean ) {
    getCommandFactory().getPropertiesCommand().execute();
  }

  @Override
  protected int getBeanIndex( @NotNull T bean ) {
    return getBeanManager().indexOf( bean );
  }

  @Override
  @Nullable
  public T getSelectedBean() {
    int index = getTable().getSelectionModel().getMinSelectionIndex();
    if ( index < 0 ) {
      return null;
    }
    return getBeanManager().getBean( index );
  }

  @Override
  protected void registerLocalCommandExecutors( @NotNull PageComponentContext context ) {
    context.register( GlobalCommandIds.DELETE, getCommandFactory().getDeleteCommand() );
    context.register( GlobalCommandIds.PROPERTIES, getCommandFactory().getPropertiesCommand() );
  }

  @Override
  @NotNull
  public Class<T> getBeanType() {
    if ( beanType == null ) {
      throw new IllegalStateException( "No BeanType has been set" );
    }
    return beanType;
  }

  public void setBeanType( @NotNull Class<T> beanType ) {
    this.beanType = beanType;
  }

  @NotNull
  public BeanManager<T> getBeanManager() {
    if ( beanManager == null ) {
      throw new IllegalStateException( "No BeanManager has been set" );
    }
    return beanManager;
  }

  public void setBeanManager( @NotNull BeanManager<T> beanManager ) {
    this.beanManager = beanManager;
  }

  @NotNull
  public BeanListCommandFactory getCommandFactory() {
    if ( commandFactory == null ) {
      commandFactory = new DefaultBeanListCommandFactory<T>( this );
    }
    return commandFactory;
  }

  public void setCommandFactory( @NotNull BeanListCommandFactory commandFactory ) {
    this.commandFactory = commandFactory;
  }

  @NotNull
  public BeanPopupMenuCreator<T> getBeanPopupMenuCreator() {
    if ( beanPopupMenuCreator == null ) {
      beanPopupMenuCreator = new BeanPopupMenuCreator<T>() {
        public JPopupMenu createMenu( @NotNull BeanListView<T> beanListView ) {
          CommandGroup commandGroup = getWindowCommandManager().createCommandGroup( getBeanType().getName() + ".tablePopupGroup", new Object[0] );
          commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getNewCommand() ) );
          if ( isSelectionNotEmpty() ) {
            commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getPropertiesCommand() ) );
            commandGroup.add( getWindowCommandManager().configure( getCommandFactory().getDeleteCommand() ) );
          }
          return commandGroup.createPopupMenu();
        }
      };
    }
    return beanPopupMenuCreator;
  }

  public void setBeanPopupMenuCreator( @NotNull BeanPopupMenuCreator<T> beanPopupMenuCreator ) {
    this.beanPopupMenuCreator = beanPopupMenuCreator;
  }

  @NotNull
  public Map<Class<?>, TableCellRenderer> getDefaultTableCellRenderers() {
    return Collections.unmodifiableMap( defaultTableCellRenderers );
  }

  public void setDefaultTableCellRenderers( @NotNull Map<String, TableCellRenderer> defaultTableCellRenderers ) {
    for ( Map.Entry<String, TableCellRenderer> entry : defaultTableCellRenderers.entrySet() ) {
      try {
        this.defaultTableCellRenderers.put( Class.forName( entry.getKey() ), entry.getValue() );
      } catch ( ClassNotFoundException e ) {
        throw new RuntimeException( e );
      }
    }
  }

  @Override
  @NotNull
  public TableModel getTableModel() {
    if ( tableModel == null ) {
      throw new IllegalStateException( "No TableModel has been set" );
    }
    return tableModel;
  }

  public void setTableModel( @NotNull TableModel tableModel ) {
    this.tableModel = tableModel;
  }
}
